require 'spec_helper'

describe 'Add response' do

  let(:server){Server.new}
  let(:default_uri) {URI('http://127.0.0.1:4567')}
  let(:default_response) {'Default Response'}

  before(:each) do
    server.start({:port => 4567})
  end

  after(:each) do
    Sinatra::Base.reset!
  end

  it 'should be able to get the response from default path' do
    server.add(default_response)
    res = Net::HTTP.get(default_uri)
    expect(res).to eq(default_response)
  end

  it 'should be able to return different for same route' do
    server.add('Response One')
    server.add('Response Two')
    res = Net::HTTP.get(default_uri)

    expect(res).to eq('Response Two')
  end

  it 'should be able to post and get the response from default path' do
    server.add(default_response,ResponseOptions.new { |opts| opts.method = :post })
    res = Net::HTTP.post_form(default_uri, {})
    expect(res.body).to eq(default_response)
  end

  it 'should be able to get the response from specific path' do
    expected_path = '/expectedPath.html'
    server.add(default_response)
    server.add('Expected Response', ResponseOptions.new{|opts| opts.path = expected_path})

    default_uri.path = expected_path
    expected_response = Net::HTTP.get_response(default_uri)

    expect(expected_response.body).to eq('Expected Response')
  end

  it 'should set response content type' do
    server.add('')
    res = Net::HTTP.get_response(default_uri)
    expect(res['Content-Type']).to eq('text/html;charset=utf-8')
  end

  it 'should be able to override response content type' do
    content_type = 'text/json;charset=utf-8'
    server.add('{name: json}', ResponseOptions.new { |opts| opts.content_type = content_type })

    res = Net::HTTP.get_response(default_uri)

    expect(res['Content-Type']).to include(content_type)
  end

  describe 'with request method of post' do
    it 'should receive a default HTTP Status of OK' do
      server.add('Expected Response', ResponseOptions.new { |opts| opts.method = :post })

      res = Net::HTTP.post_form(default_uri, {})

      expect(res).to be_a(Net::HTTPOK)
    end

    it 'should be able to override status' do
      opts = ResponseOptions.new do |opts|
        opts.method = :post
        opts.status = 500
      end

      server.add('Expected Response', opts)
      res = Net::HTTP.post_form(default_uri, {})
      expect(res).to be_a(Net::HTTPInternalServerError)
    end
  end

end
