require 'spec_helper'

describe 'Wait for next request' do

  let(:server){Server.new}
  let(:default_uri) {URI('http://127.0.0.1:4567')}
  let(:default_response) {'Default Response'}

  before(:each) do
    server.start({:port => 4567})
  end

  after(:each) do
    Sinatra::Base.reset!
    server.clear_requests
  end

  it 'should return the first request' do
    server.add(default_response)
    thread = Thread.new do
      Thread.current[:result]= server.wait_for_next_request
    end
    Net::HTTP.get(default_uri)
    thread.join
    expect(thread[:result].env['REQUEST_URI']).to include default_uri.to_s
  end

  it 'should wait for a new request' do
    server.add(default_response)
    server.add(default_response, ResponseOptions.new{|opts| opts.method= :post; opts.path= '/Test.html'})

    Net::HTTP.get(default_uri)
    thread = Thread.new do
      Thread.current[:result]= server.wait_for_next_request
    end
    Net::HTTP.post_form(default_uri + '/Test.html', {:test => 'data'})
    thread.join
    expect(thread[:result].env['REQUEST_METHOD']).to eq "POST"
  end

  it 'should default timeout value to 5' do
    t1 = Time.now

    request = server.wait_for_next_request
    t2 = Time.now

    expect(request).to be_nil
    time = (t2 - t1)
    expect(time.to_i).to be 5
  end

  it 'should allow the timeout value to be set' do
    t1 = Time.now

    request = server.wait_for_next_request 2
    t2 = Time.now

    expect(request).to be_nil
    expect((t2 - t1).to_i).to be 2
  end

end